@@ -22,23 +22,22 @@ use strict;
 use warnings;
 use Data::Dumper;
 use Term::ReadLine;
-use Term::ANSIColor   qw( colored );
-use PadWalker         qw( peek_my  peek_our );
-use Scalar::Util      qw( blessed reftype );
-use Module::Functions qw( get_full_functions );
-use Class::Tiny       qw( term attr debug );
-use feature           qw( say state );
-use parent            qw( Exporter );
-use subs              qw( p uniq );
+use Term::ANSIColor qw( colored );
+use PadWalker       qw( peek_my  peek_our );
+use Scalar::Util    qw( blessed reftype );
+use Class::Tiny     qw( term attr debug );
+use feature         qw( say state );
+use parent          qw( Exporter );
+use subs            qw( p uniq );
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 our @EXPORT  = qw( run p );
 
 =head1 NAME
 
-Runtime::Debugger - Easy to use REPL with existing lexicals support.
+Runtime::Debugger - Easy to use REPL with existing lexical support.
 
-(emphasis on "existing" since I have not yet found this support in others modules).
+(emphasis on "existing" since I have not yet found this support in other modules).
 
 =head1 SYNOPSIS
 
@@ -104,7 +103,7 @@ Sample Output:
  $ print $our_var
  Compile error: Global symbol "$our_var" requires explicit package name ...
 
-=head2 Devel::REPL
+=head2 Reply
 
 This module also looked nice, but same issue.
 
@@ -234,6 +233,7 @@ sub _init {
     #
     # TODO: After testing is setup, try removing ">$@".
     $attribs->{completer_word_break_characters} =~ s/ [>] //xg;
+    $attribs->{completer_word_break_characters} .= '[';    # $my->[
     $attribs->{special_prefixes} = '$@%&';
 
     # Build the debugger object.
@@ -251,7 +251,7 @@ sub _init {
 
     $self->_restore_history;
 
-    # Setup some signal hnndling.
+    # Setup some signal handling.
     for my $signal ( qw( INT TERM HUP ) ) {
         $SIG{$signal} = sub { $self->_exit( $signal ) };
     }
@@ -259,6 +259,41 @@ sub _init {
     $self;
 }
 
+sub _step {
+    my ( $self ) = @_;
+
+    if ( not $self->{vars_all} ) {
+        $self->_setup_vars;
+        $self->help;    # Show help when first loading the debugger.
+    }
+
+    my $input = $self->term->readline( "perl>" ) // '';
+    say "input_after_readline=[$input]" if $self->debug;
+
+    # Change '#1' to '--maxdepth=1'
+    if ( $input =~ / ^ p\b /x ) {
+        $input =~ s/
+            \s*
+            \#(\d)     #2 to --maxdepth=2
+            \s*
+        $ /, '--maxdepth=$1'/x;
+    }
+
+    # Change "COMMAND ARG" to "$repl->COMMAND(ARG)".
+    $input =~ s/ ^
+        (
+              help
+            | hist
+        ) \b
+        (.*)
+    $ /\$repl->$1($2)/x;
+
+    $self->_exit( $input ) if $input eq 'q';
+
+    say "input_after_step=[$input]" if $self->debug;
+    $input;
+}
+
 # Completion
 
 sub _complete {
@@ -284,7 +319,16 @@ sub _complete {
 
     # Hash or hashref - Show possible keys and string variables.
     return $self->_complete_hash( "$1", @_ )
-      if substr( $line, 0, $end ) =~ / (\S+)->\{ [^}]* $ /x;
+      if substr( $line, 0, $end ) =~ /
+        (
+            [\$}@%]                 # Variable sigil.
+            (?:
+                (?!->|\s).)+        # Next if not a -> or space.
+        )
+        (?:->)?                     # Maybe a ->.
+        \{                          # Opening brace.
+        [^}]*                       # Any non braces.
+    $ /x;
 
     # Otherwise assume its a variable.
     return $self->_complete_vars( @_ );
@@ -340,12 +384,13 @@ sub _complete_arrow {
 
         my $methods = $self->{methods}{$obj_or_coderef};
         if ( not $methods ) {
-            $methods = [ get_full_functions( ref $obj_or_coderef ) ];
+            $methods = $self->_get_object_functions( $obj_or_coderef );
             $self->{methods}{$obj_or_coderef} = $methods;
 
             # push @$methods, "(";    # Access as method or hash refs.
             push @$methods, "{" if reftype( $obj_or_coderef ) eq "HASH";
             push @$methods, @{ $self->{vars_string} };
+            @$methods = uniq sort @$methods;
 
             # push @$methods, $self->{vars_all}; # TODO: Add scalars.
         }
@@ -369,22 +414,67 @@ sub _complete_arrow {
         );
     }
 
+    # Hashref.
+    if ( ref( $obj_or_coderef ) eq "HASH" ) {
+        say "IS_HASH $obj_or_coderef" if $self->debug;
+        return $self->_match(
+            words   => ["{"],
+            prepend => "$text",
+            nospace => 1,
+        );
+    }
+
+    # Arrayref.
+    if ( ref( $obj_or_coderef ) eq "ARRAY" ) {
+        say "IS_ARRAY: $obj_or_coderef" if $self->debug;
+        return $self->_match(
+            words   => ["["],
+            prepend => "$text",
+            nospace => 1,
+        );
+    }
+
     say "NOT OBJECT or CODEREF: $obj_or_coderef" if $self->debug;
     return;
 }
 
+sub _get_object_functions {
+    my ( $self, $obj ) = @_;
+    my $class = ref $obj;
+    no strict 'refs';
+
+    my @functions = grep {
+        !/ ^
+        (?:
+              BEGIN
+            | UNITCHECK
+            | INIT
+            | CHECK
+            | END
+            | import
+            | AUTOLOAD
+            | DESTROY
+        )
+        $ /x
+      }
+      sort
+      keys %{"${class}::"};
+
+    \@functions;
+}
+
 sub _complete_hash {
     my $self = shift;
     my ( $var, $text, $line, $start, $end ) = @_;
     $self->_dump_args( @_ ) if $self->debug;
 
     my @hash_keys = @{ $self->{vars_string} };
-    my $ref       = $self->{peek_all}{$var};
+    my $ref       = $self->{peek_all}{$var} // '';
     $ref = $$ref if reftype( $ref ) eq "REF";
     push @hash_keys, keys %$ref if reftype( $ref ) eq "HASH";
 
     $self->_match(
-        words   => \@hash_keys,
+        words   => [ sort @hash_keys ],
         partial => $text,
         nospace => 1,
     );
@@ -420,10 +510,15 @@ sub _match {
     my %parms = @_;
     $self->_dump_args( @_ ) if $self->debug;
 
+    # completion_word does not automationally get reset per call.
+    # completion_suppress_append gets reset perl call.
+    # attempted_completion_over gets reset perl call.
     $parms{partial} //= "";
     $parms{prepend} //= "";
     $self->attr->{completion_word}            = $parms{words};
     $self->attr->{completion_suppress_append} = 1 if $parms{nospace};
+    $self->attr->{attempted_completion_over} =
+      1;    # Will not use filename completion at all.
 
     map { "$parms{prepend}$_" }
       $self->term->completion_matches( $parms{partial},
@@ -450,47 +545,68 @@ sub _define_commands {
 sub _setup_vars {
     my ( $self ) = @_;
 
-    # Get and cache the current variables in the invoking scope.
-    #
-    # Note: this block was originally in _step since the intent
-    # was to be able to see newly added lexicals or globals.
-    # (which does not seem to be possible since the lexical scope
-    # of a new variable would be found to the eval block).
-    #
-    # Although globals can be created, they would appear in the
-    # invoking scope (like "main").
-    #
-    # Moved here to be able to capture the debugger object "$repl".
+    $self->_set_peeks;
+    $self->_split_vars_by_types;
+
+    $self->{commands} = [ $self->_define_commands ];
+    $self->{commands_and_vars_all} =
+      [ @{ $self->{commands} }, @{ $self->{vars_all} } ];
+
+    $self->_normalize_var_defaults;
+
+    $self;
+}
+
+sub _set_peeks {
+    my ( $self ) = @_;
+
+    # Get the current variables in the invoking scope.
     #
     # CAUTION: avoid having the same name for a lexical and global
     # variable since the last variable declared would "win".
-    my $levels       = 2;    # How many levels until at "$repl=" or main.
+
+    my $levels       = 3;    # How many levels until at "$repl=" or main.
     my $peek_my      = peek_my( $levels );
     my $peek_our     = peek_our( $levels );
     my %peek_all     = ( %$peek_our, %$peek_my );
-    my @vars_lexical = sort keys %$peek_my;
-    my @vars_global  = sort keys %$peek_our;
-    my @vars_all     = sort uniq @vars_lexical, @vars_global;
+    my @vars_lexical = keys %$peek_my;
+    my @vars_global  = keys %$peek_our;
+    my @vars_all     = uniq @vars_lexical, @vars_global;
 
-    # Cache variables.
     $self->{peek_my}      = $peek_my;
     $self->{peek_our}     = $peek_our;
     $self->{peek_all}     = \%peek_all;
     $self->{vars_lexical} = \@vars_lexical;
     $self->{vars_global}  = \@vars_global;
     $self->{vars_all}     = \@vars_all;
+}
 
-    my @commands = $self->_define_commands;
-
-    $self->{commands}              = \@commands;
-    $self->{commands_and_vars_all} = [ sort( @commands, @vars_all ) ];
+sub _split_vars_by_types {
+    my ( $self ) = @_;
+    my @queue = @{ $self->{vars_all} // [] };
 
     # Separate variables by types.
-    for ( @vars_all ) {
+    my %already_stored;
+    my %added_duplicates;    # These are ok to have twice.
+
+    while ( local $_ = shift @queue ) {
+
+        # Show duplcate variables with same name (probably different sigil).
+        if ( $already_stored{$_} ) {
+            if ( $self->debug or not $added_duplicates{$_} ) {
+                $self->_show_error(
+                        "Skipping variable with same name: '$_' "
+                      . "(maybe same name, but different sigil?)" );
+            }
+            next;
+        }
+
+        $already_stored{$_}++;    # First time seeing it then.
+
         if ( / ^ \$ /x ) {
             push @{ $self->{vars_scalar} }, $_;
 
-            my $ref  = $peek_all{$_};
+            my $ref  = $self->{peek_all}{$_};
             my $type = ref( $ref );
             if ( $type eq "SCALAR" ) {
                 push @{ $self->{vars_string} }, $_;
@@ -519,51 +635,71 @@ sub _setup_vars {
         }
         elsif ( / ^ \@ /x ) {
             push @{ $self->{vars_array} }, $_;
+
+            # Allow access via @array, $array[0]
+            my $var_scalar = '$' . substr( $_, 1 );
+            push @{ $self->{vars_all} }, $var_scalar;
+            push @queue,                 $var_scalar;
+            $self->{peek_all}{$var_scalar} =
+              $self->{peek_all}{$_};
+            $added_duplicates{$var_scalar}++;
+
+            say "Added $_: $var_scalar" if $self->debug;
         }
         elsif ( / ^ % /x ) {
             push @{ $self->{vars_hash} }, $_;
+
+            # Allow access via %hash, $hash{key} and @hash{key,key}
+            my $var_scalar = '$' . substr( $_, 1 );
+            my $var_array  = '@' . substr( $_, 1 );
+            push @{ $self->{vars_all} }, $var_scalar, $var_array;
+            push @queue,                 $var_scalar, $var_array;
+            $self->{peek_all}{$var_scalar} = $self->{peek_all}{$var_array} =
+              $self->{peek_all}{$_};
+            $added_duplicates{$var_scalar}++;
+            $added_duplicates{$var_array}++;
+
+            say "Added $_: $var_scalar, $var_array" if $self->debug;
         }
         else {
             push @{ $self->{vars_else} }, $_;
         }
     }
-
-    $self;
 }
 
-sub _step {
+sub _normalize_var_defaults {
     my ( $self ) = @_;
 
-    if ( not $self->{vars_all} ) {
-        $self->_setup_vars;
-        $self->help;    # Show help when first loading the debugger.
+    my @vars = qw(
+      vars_lexical
+      vars_global
+      vars_all
+
+      vars_scalar
+      vars_string
+      vars_ref
+      vars_obj
+      vars_code
+      vars_hashref
+      vars_arrayref
+      vars_ref_else
+      vars_scalar_else
+
+      vars_array
+
+      vars_hash
+
+      vars_else
+
+      commands
+      commands_and_vars_all
+    );
+
+    # Make sure all "vars_*" are defined, sorted, and uniq:
+    for ( @vars ) {
+        $self->{$_} = [ uniq sort @{ $self->{$_} // [] } ];
     }
 
-    my $input = $self->term->readline( "perl>" ) // '';
-    say "input_after_readline=[$input]" if $self->debug;
-
-    # Change '#1' to '--maxdepth=1'
-    if ( $input =~ / ^ p\b /x ) {
-        $input =~ s/
-            \s*
-            \#(\d)     #2 to --maxdepth=2
-            \s*
-        $ /, '--maxdepth=$1'/x;
-    }
-
-    # Change "COMMAND ARG" to "$repl->COMMAND(ARG)".
-    $input =~ s/ ^
-        (
-              help
-            | hist
-        ) \b
-        (.*)
-    $ /\$repl->$1($2)/x;
-
-    $self->_exit( $input ) if $input eq 'q';
-
-    say "input_after_step=[$input]" if $self->debug;
-    $input;
 }
 
 # Help
